@@ -45,7 +45,7 @@ class Ngpf(object):
         self._api._remove(ixn_topology, topologies.values())
         for device in devices:
             ixn_topology.find(
-                Name=self._api._get_topology_name(device.container_name))
+                Name='^%s$' % self._api._get_topology_name(device.container_name))
             if len(ixn_topology) > 0:
                 self._api._remove(ixn_topology.DeviceGroup, [device])
         for device in devices:
@@ -53,7 +53,7 @@ class Ngpf(object):
                 'Name': self._api._get_topology_name(device.container_name),
                 'Ports': [self._api.ixn_objects[device.container_name]]
             }
-            ixn_topology.find(Name=args['Name'])
+            ixn_topology.find(Name='^%s$' % args['Name'])
             if len(ixn_topology) == 0:
                 ixn_topology.add(**args)
             else:
@@ -66,7 +66,7 @@ class Ngpf(object):
         One /topology/deviceGroup for every device in port.devices 
         """
         args = {'Name': device.name, 'Multiplier': device.device_count}
-        ixn_device_group.find(Name=device.name)
+        ixn_device_group.find(Name='^%s$' % device.name)
         if len(ixn_device_group) == 0:
             ixn_device_group.add(**args)[-1]
         else:
@@ -111,7 +111,7 @@ class Ngpf(object):
         args = {
             'Name': ethernet.name,
         }
-        ixn_ethernet.find(Name=ethernet.name)
+        ixn_ethernet.find(Name='^%s$' % ethernet.name)
         if len(ixn_ethernet) == 0:
             ixn_ethernet.add(**args)
         else:
@@ -146,7 +146,7 @@ class Ngpf(object):
         args = {
             'Name': ipv4.name,
         }
-        ixn_ipv4.find(Name=ipv4.name)
+        ixn_ipv4.find(Name='^%s$' % ipv4.name)
         if len(ixn_ipv4) == 0:
             ixn_ipv4.add(**args)[-1]
         else:
@@ -162,7 +162,7 @@ class Ngpf(object):
         args = {
             'Name': ipv6.name,
         }
-        ixn_ipv6.find(Name=ipv6.name)
+        ixn_ipv6.find(Name='^%s$' % ipv6.name)
         if len(ixn_ipv6) == 0:
             ixn_ipv6.add(**args)[-1]
         else:
@@ -178,7 +178,7 @@ class Ngpf(object):
         args = {
             'Name': bgpv4.name,
         }
-        ixn_bgpv4.find(Name=bgpv4.name)
+        ixn_bgpv4.find(Name='^%s$' % bgpv4.name)
         if len(ixn_bgpv4) == 0:
             ixn_bgpv4.add(**args)[-1]
         else:
