@@ -1,5 +1,4 @@
 import json
-from jsonpath_ng.ext import parse
 import time
 import re
 from ixnetwork_open_traffic_generator.timer import Timer
@@ -8,17 +7,34 @@ from ixnetwork_open_traffic_generator.timer import Timer
 class Vport(object):
     """Transforms OpenAPI objects into IxNetwork objects
 
-    - Port to /vport
-    - Port.Capture to /vport/capture
-    - Port.Layer1 to /vport/l1Config/...
-
-    Uses resourcemanager to set the vport location and l1Config as it is the
-    most efficient way.  
-    DO NOT use the AssignPorts API as it is too slow. 
-
     Args
     ----
     - ixnetworkapi (IxNetworkApi): instance of the IxNetworkApi class
+
+    Transformations
+    ---------------
+    - /components/schemas/Port to /vport
+    - /components/schemas/Capture to /vport/capture
+    - /components/schemas/Layer1 to /vport/l1Config/...
+
+    Process
+    -------
+    - Remove any vports that are not in the config.ports
+    - Add any vports that are in the config.ports
+    - If the location of the config.ports.location is different than the
+      the /vport -connectedTo property set it to None
+    - If the config.ports.location is None don't connect the ports
+      else connect the port, get the vport type, set the card mode based on the
+      config.layer1.speed
+
+    Notes
+    -----
+    - Uses resourcemanager to set the vport location and l1Config as it is the
+      most efficient way. DO NOT use the AssignPorts API as it is too slow. 
+    - Only setup l1Config if location is connected. 
+    - Given a connected location and speed the vport -type, card resource mode
+      and l1Config sub node are derived.
+
     """
     _SPEED_MAP = {
         'speed_100_gbps': 'speed100g',
@@ -32,6 +48,35 @@ class Vport(object):
         'speed_10_fd_mbps': 'speed10fd',
         'speed_10_hd_mbps': 'speed10hd'
     }
+    _VM_SPEED_MAP = {
+        'speed_400_gbps': 'speed400g',
+        'speed_200_gbps': 'speed200g',
+        'speed_100_gbps': 'speed100g',
+        'speed_90_gbps': 'speed90g',
+        'speed_80_gbps': 'speed80g',
+        'speed_70_gbps': 'speed70g',
+        'speed_60_gbps': 'speed60g',
+        'speed_50_gbps': 'speed50g',
+        'speed_40_gbps': 'speed40g',
+        'speed_30_gbps': 'speed30g',
+        'speed_25_gbps': 'speed25g',
+        'speed_20_gbps': 'speed20g',
+        'speed_10_gbps': 'speed10g',
+        'speed_9_gbps': 'speed9000',
+        'speed_8_gbps': 'speed8000',
+        'speed_7_gbps': 'speed7000',
+        'speed_6_gbps': 'speed6000',
+        'speed_5_gbps': 'speed5000',
+        'speed_4_gbps': 'speed4000',
+        'speed_3_gbps': 'speed3000',
+        'speed_2_gbps': 'speed2000',
+        'speed_1_gbps': 'speed1000',
+        'speed_100_mbps': 'speed100',
+        'speed_100_fd_mbps': 'speed100',
+        'speed_100_hd_mbps': 'speed100',
+        'speed_10_fd_mbps': 'speed100',
+        'speed_10_hd_mbps': 'speed100'
+    }
     _ADVERTISE_MAP = {
         'advertise_one_thousand_mbps': 'speed1000',
         'advertise_one_hundred_fd_mbps': 'speed100fd',
@@ -43,14 +88,24 @@ class Vport(object):
         'ieee_802_1qbb': 'ieee802.1Qbb',
         'ieee_802_3x': 'ieee_802_3x'
     }
+
     _RESULT_COLUMNS = [
-        'name', 'location', 'link', 'capture', 'frames_tx', 'frames_rx',
-        'frames_tx_rate', 'frames_rx_rate', 'bytes_tx', 'bytes_rx',
-        'bytes_tx_rate', 'bytes_rx_rate', 'pfc_class_0_frames_rx',
-        'pfc_class_1_frames_rx', 'pfc_class_2_frames_rx',
-        'pfc_class_3_frames_rx', 'pfc_class_4_frames_rx',
-        'pfc_class_5_frames_rx', 'pfc_class_6_frames_rx',
-        'pfc_class_7_frames_rx'
+        ('frames_tx', 'Frames Tx.', int),
+        ('frames_rx', 'Valid Frames Rx.', int),
+        ('frames_tx_rate', 'Frames Tx. Rate', float),
+        ('frames_rx_rate', 'Valid Frames Rx. Rate', float),
+        ('bytes_tx', 'Bytes Tx.', int),
+        ('bytes_rx', 'Bytes Rx.', int),
+        ('bytes_tx_rate', 'Bytes Tx. Rate', float),
+        ('bytes_rx_rate', 'Bytes Rx. Rate', float),
+        ('pfc_class_0_frames_rx', 'Rx Pause Priority Group 0 Frames', int),
+        ('pfc_class_1_frames_rx', 'Rx Pause Priority Group 1 Frames', int),
+        ('pfc_class_2_frames_rx', 'Rx Pause Priority Group 2 Frames', int),
+        ('pfc_class_3_frames_rx', 'Rx Pause Priority Group 3 Frames', int),
+        ('pfc_class_4_frames_rx', 'Rx Pause Priority Group 4 Frames', int),
+        ('pfc_class_5_frames_rx', 'Rx Pause Priority Group 5 Frames', int),
+        ('pfc_class_6_frames_rx', 'Rx Pause Priority Group 6 Frames', int),
+        ('pfc_class_7_frames_rx', 'Rx Pause Priority Group 7 Frames', int),
     ]
 
     def __init__(self, ixnetworkapi):
@@ -59,8 +114,8 @@ class Vport(object):
     def config(self):
         """Transform config.ports into Ixnetwork.Vport
         1) delete any vport that is not part of the config
-        2) create a vport for every config.ports[] that is not present in IxNetwork
-        3) set config.ports[].location to /vport -location using resourcemanager
+        2) create a vport for every config.ports[] not present in IxNetwork
+        3) set config.ports[].location to /vport -location or -connectedTo
         4) set /vport/l1Config/... properties using the corrected /vport -type
         5) connectPorts to use new l1Config settings and clearownership
         """
@@ -78,7 +133,9 @@ class Vport(object):
 
     def _import(self, imports):
         if len(imports) > 0:
-            self._resource_manager.ImportConfig(json.dumps(imports), False)
+            for errata in self._resource_manager.ImportConfig(
+                    json.dumps(imports), False):
+                self._api.info(errata)
 
     def _delete_vports(self):
         """Delete any vports from the api server that do not exist in the new config
@@ -102,6 +159,7 @@ class Vport(object):
                 location = getattr(port, 'location', None)
                 if location is None:
                     vport_import['connectedTo'] = location
+                    port.location = None
                 imports.append(vport_import)
         self._import(imports)
         for name, vport in self._api.select_vports().items():
@@ -115,7 +173,8 @@ class Vport(object):
         imports = []
         vports = self._api.select_vports()
         for vport in vports.values():
-            if vport['capture']['hardwareEnabled'] is True or vport['capture']['softwareEnabled'] is True:
+            if vport['capture']['hardwareEnabled'] is True or vport['capture'][
+                    'softwareEnabled'] is True:
                 capture = {
                     'xpath': vport['capture']['xpath'],
                     'captureMode': 'captureTriggerMode',
@@ -176,22 +235,25 @@ class Vport(object):
             location = getattr(port, 'location', None)
             if location is not None and ';' in location:
                 chassis_address = location.split(';')[0]
-                chassis.find(Hostname='^(%s)$' % chassis_address)
+                chassis.find(Hostname='^%s$' % chassis_address)
                 if len(chassis) == 0:
                     add_addresses.append(chassis_address)
                 check_addresses.append(chassis_address)
         add_addresses = set(add_addresses)
         check_addresses = set(check_addresses)
         if len(add_addresses) > 0:
-            with Timer(self._api, 'Add location hosts [%s]' % ', '.join(add_addresses)):      
+            with Timer(self._api,
+                       'Add location hosts [%s]' % ', '.join(add_addresses)):
                 for add_address in add_addresses:
                     chassis.add(Hostname=add_address)
         if len(check_addresses) > 0:
-            with Timer(self._api, 'Location hosts check [%s]' % ', '.join(check_addresses)):      
+            with Timer(
+                    self._api,
+                    'Location hosts check [%s]' % ', '.join(check_addresses)):
                 start_time = time.time()
                 while True:
                     chassis.find(Hostname='^(%s)$' % '|'.join(check_addresses),
-                                State='^ready$')
+                                 State='^ready$')
                     if len(chassis) == len(check_addresses):
                         break
                     if time.time() - start_time > HostReadyTimeout:
@@ -203,8 +265,9 @@ class Vport(object):
     def _set_location(self):
         location_supported = True
         try:
-            self._api._ixnetwork._connection._options(self._api._ixnetwork.href + '/locations')
-        except:
+            self._api._ixnetwork._connection._options(
+                self._api._ixnetwork.href + '/locations')
+        except Exception:
             location_supported = False
 
         locations = []
@@ -242,7 +305,8 @@ class Vport(object):
         self._clear_ownership(clear_locations)
         with Timer(self._api, 'Location connect [%s]' % ', '.join(locations)):
             self._import(imports)
-        with Timer(self._api, 'Location state check [%s]' % ', '.join(locations)):
+        with Timer(self._api,
+                   'Location state check [%s]' % ', '.join(locations)):
             self._api._vport.find(ConnectionState='^(?!connectedLink).*$')
             if len(self._api._vport) > 0:
                 self._api._vport.ConnectPorts()
@@ -250,16 +314,19 @@ class Vport(object):
             timeout = 30
             while True:
                 self._api._vport.find(Name='^(%s)$' % '|'.join(locations),
-                                    ConnectionState='^connectedLink')
+                                      ConnectionState='^connectedLink')
                 if len(self._api._vport) == len(locations):
                     break
                 if time.time() - start > timeout:
                     raise RuntimeError(
-                        'After %s seconds, not all locations [%s] are reachable' %
-                        (timeout, ', '.join([vport.Name for vport in self._api._vport])))
+                        'After %s seconds, not all locations [%s] are reachable'
+                        % (timeout, ', '.join(
+                            [vport.Name for vport in self._api._vport])))
                 time.sleep(2)
-            for vport in self._api._vport.find(ConnectionState='^(?!connectedLinkUp).*$'):
-                self._api.warning('%s %s' % (vport.Name, vport.ConnectionState))
+            for vport in self._api._vport.find(
+                    ConnectionState='^(?!connectedLinkUp).*$'):
+                self._api.warning('%s %s' %
+                                  (vport.Name, vport.ConnectionState))
 
     def _set_layer1(self):
         """Set the /vport/l1Config/... properties
@@ -271,14 +338,24 @@ class Vport(object):
         if self._api.config.layer1 is None:
             return
         vports = self._api.select_vports()
+        # set the vport type first and commit it otherwise some settings
+        # such as l1config/... will not be committed properly
         imports = []
         reset_auto_negotiation = dict()
         for layer1 in self._api.config.layer1:
             for port_name in layer1.port_names:
-                vport = vports[port_name]
-                self._set_l1config_properties(vport, layer1, imports)
+                self._set_vport_type(vports[port_name], layer1, imports)
         self._import(imports)
-        # Due to dependency attribute (ieeeL1Defaults) resetting enableAutoNegotiation
+        # set the resource mode changes and any other l1config/... changes
+        # and commit them
+        imports = []
+        for layer1 in self._api.config.layer1:
+            for port_name in layer1.port_names:
+                self._set_l1config_properties(vports[port_name], layer1,
+                                              imports)
+        self._import(imports)
+        # Due to dependency attribute (ieeeL1Defaults)
+        # reset enableAutoNegotiation
         imports = []
         for layer1 in self._api.config.layer1:
             for port_name in layer1.port_names:
@@ -295,7 +372,6 @@ class Vport(object):
                 'connectedLinkUp', 'connectedLinkDown'
         ]:
             return
-        self._set_vport_type(vport, layer1, imports)
         self._set_card_resource_mode(vport, layer1, imports)
         self._set_auto_negotiation(vport, layer1, imports)
 
@@ -323,7 +399,8 @@ class Vport(object):
                     break
         if aggregation_mode is not None and aggregation_mode != card[
                 'aggregationMode']:
-            self._api.info('Setting %s layer1 mode' % aggregation_mode)
+            self._api.info('Setting %s to resource mode %s' %
+                           (card['description'], aggregation_mode))
             imports.append({
                 'xpath': card['xpath'],
                 'aggregationMode': aggregation_mode
@@ -351,7 +428,8 @@ class Vport(object):
         elegible_fcoe_vport_types = [
             'ethernet', 'tenGigLan', 'fortyGigLan', 'tenGigWan',
             'hundredGigLan', 'tenFortyHundredGigLan', 'novusHundredGigLan',
-            'novusTenGigLan', 'krakenFourHundredGigLan', 'aresOneHundredGigLan'
+            'novusTenGigLan', 'krakenFourHundredGigLan',
+            'aresOneFourHundredGigLan', 'starFourHundredGigLan'
         ]
         if fcoe is True and vport_type in elegible_fcoe_vport_types:
             vport_type = vport_type + 'Fcoe'
@@ -382,7 +460,7 @@ class Vport(object):
             'xpath':
             vport['xpath'] + '/l1Config/' + vport['type'].replace('Fcoe', ''),
             'speed':
-            Vport._SPEED_MAP[layer1.speed],
+            self._get_speed(vport, layer1),
             'media':
             layer1.media,
             'autoNegotiate':
@@ -410,12 +488,23 @@ class Vport(object):
             layer1.ieee_media_defaults,
             'speed':
             Vport._SPEED_MAP[layer1.speed],
-            'enableAutoNegotiation': layer1.auto_negotiate,
-            'enableRsFec': None if layer1.auto_negotiation is None else layer1.auto_negotiation.rs_fec,
-            'linkTraining': None if layer1.auto_negotiation is None else layer1.auto_negotiation.link_training
+            'enableAutoNegotiation':
+            False if layer1.auto_negotiate is None else layer1.auto_negotiate,
+            'enableRsFec':
+            False if layer1.auto_negotiation is None else
+            layer1.auto_negotiation.rs_fec,
+            'linkTraining':
+            False if layer1.auto_negotiation is None else
+            layer1.auto_negotiation.link_training
         }
         self._add_l1config_import(vport, proposed_import, imports)
 
+    def _get_speed(self, vport, layer1):
+        if vport['type'] == 'ethernetvm':
+            return Vport._VM_SPEED_MAP[layer1.speed]
+        else:
+            return Vport._SPEED_MAP[layer1.speed]
+
     def _reset_auto_negotiation(self, vport, layer1, imports):
         if layer1.speed.endswith(
                 '_mbps') is False and layer1.speed != 'speed_1_gbps':
@@ -432,7 +521,8 @@ class Vport(object):
             pfc = flow_control.ieee_802_1qbb
             fcoe = {
                 'xpath':
-                vport['xpath'] + '/l1Config/' + vport['type'].replace('Fcoe', '') + '/fcoe',
+                vport['xpath'] + '/l1Config/' +
+                vport['type'].replace('Fcoe', '') + '/fcoe',
                 'enablePFCPauseDelay':
                 False if pfc.pfc_delay == 0 else True,
                 'flowControlType':
@@ -459,7 +549,7 @@ class Vport(object):
     def _clear_ownership(self, locations):
         try:
             force_ownership = self._api.config.options.port_options.location_preemption
-        except:
+        except Exception:
             force_ownership = False
         if force_ownership is True:
             available_hardware_hrefs = {}
@@ -467,15 +557,18 @@ class Vport(object):
             for location in locations:
                 if ';' in location:
                     clp = location.split(';')
-                    chassis = self._api._ixnetwork.AvailableHardware.Chassis.find(Hostname=clp[0])
+                    chassis = self._api._ixnetwork.AvailableHardware.Chassis.find(
+                        Hostname=clp[0])
                     if len(chassis) > 0:
-                        available_hardware_hrefs[location] = '%s/card/%s/port/%s' % (
-                            chassis.href, abs(int(clp[1])), abs(int(clp[2])))
+                        available_hardware_hrefs[
+                            location] = '%s/card/%s/port/%s' % (
+                                chassis.href, abs(int(clp[1])), abs(int(
+                                    clp[2])))
                 elif '/' in location:
                     appliance = location.split('/')[0]
                     locations = self._api._ixnetwork.Locations
-                    locations.find(Hostname=appliance)  
-                    if len(locations) == 0:    
+                    locations.find(Hostname=appliance)
+                    if len(locations) == 0:
                         locations.add(Hostname=appliance)
                     ports = locations.Ports.find(Location='^%s$' % location)
                     if len(ports) > 0:
@@ -492,7 +585,7 @@ class Vport(object):
             return
         try:
             row[column_name] = column_type(column_value)
-        except:
+        except Exception:
             if column_type.__name__ in ['float', 'int']:
                 row[column_name] = 0
             else:
@@ -521,51 +614,35 @@ class Vport(object):
                 port_row, 'link', 'up'
                 if vport['connectionState'] == 'connectedLinkUp' else 'down')
             self._set_result_value(port_row, 'capture', 'stopped')
+            # init all columns with corresponding zero-values so that
+            # the underlying dictionary contains all requested columns
+            # in an event of unwanted exceptions
+            for ext_name, _, typ in self._RESULT_COLUMNS:
+                self._set_result_value(port_row, ext_name, 0, typ)
+
             port_rows[vport['name']] = port_row
+
         try:
             table = self._api.assistant.StatViewAssistant('Port Statistics')
+            # TBD: REMOVE unnecessary try/except
             for row in table.Rows:
-                port_row = port_rows[row['Port Name']]
-                self._set_result_value(port_row, 'frames_tx',
-                                       row['Frames Tx.'], int)
-                self._set_result_value(port_row, 'frames_rx',
-                                       row['Valid Frames Rx.'], int)
-                self._set_result_value(port_row, 'frames_tx_rate',
-                                       row['Frames Tx. Rate'], float)
-                self._set_result_value(port_row, 'frames_rx_rate',
-                                       row['Valid Frames Rx. Rate'], float)
-                self._set_result_value(port_row, 'bytes_tx', row['Bytes Tx.'],
-                                       int)
-                self._set_result_value(port_row, 'bytes_rx', row['Bytes Rx.'],
-                                       int)
-                self._set_result_value(port_row, 'bytes_tx_rate',
-                                       row['Bytes Tx. Rate'], float)
-                self._set_result_value(port_row, 'bytes_rx_rate',
-                                       row['Bytes Rx. Rate'], float)
-                self._set_result_value(port_row, 'pfc_class_0_frames_rx',
-                                       row['Rx Pause Priority Group 0 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_1_frames_rx',
-                                       row['Rx Pause Priority Group 1 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_2_frames_rx',
-                                       row['Rx Pause Priority Group 2 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_3_frames_rx',
-                                       row['Rx Pause Priority Group 3 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_4_frames_rx',
-                                       row['Rx Pause Priority Group 4 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_5_frames_rx',
-                                       row['Rx Pause Priority Group 5 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_6_frames_rx',
-                                       row['Rx Pause Priority Group 6 Frames'],
-                                       int)
-                self._set_result_value(port_row, 'pfc_class_7_frames_rx',
-                                       row['Rx Pause Priority Group 7 Frames'],
-                                       int)
-        except:
+                # keep plugging values for next columns even if the
+                # current one raises exception
+                try:
+                    port_row = port_rows[row['Port Name']]
+                    for ext_name, int_name, typ in self._RESULT_COLUMNS:
+                        try:
+                            self._set_result_value(port_row, ext_name,
+                                                   row[int_name], typ)
+                        except Exception:
+                            # TODO print a warning maybe ?
+                            pass
+                except Exception:
+                    # TODO print a warning maybe ?
+                    pass
+
+        except Exception:
+            # TODO print a warning maybe ?
             pass
+
         return port_rows.values()
